; **************************************************************************
; Name:
;	  uStart
;
; Version:
;   1.6.0
;
; Function / Funktion:
; 	A small and easy program launcher that uses relative paths for
; 	starting applications. Thus it could be used on USB-sticks.
;   When left-clicking on the trayicon you'll see the traymenu for 
;   starting applications. When right-clicking on the trayicon the
;   traymenu for configuration and help opens.
;   The settings are saved in the file "uStart.prefs" in the same
;   directory where uStart.exe resides.
;
; Goals / Ziele:
;   - Application/Filesize should be small
;   - No fancy functions or design stuff
;   - Simple and Easy
;
; To Do / Noch zu tun:
;   - maybe: drag & drop
;
; Written with / Geschrieben mit:
;   PureBasic 5.60
;
;{ License (BSD-style) / Lizenz (BSD hnlich):
;   ENGLISH:
;           Copyright 2006-2017 Karl Loncarek
;           All rights reserved.
;
;           Redistribution and use in source and binary forms, with or without
;           modification, are permitted provided that the following conditions
;           are met:
;
;             1. Redistributions of source code must retain the above copyright
;                notice, this list of conditions and the following disclaimer.
;             2. Redistributions in binary form must reproduce the above
;                copyright notice, this list of conditions and the following
;                disclaimer in the documentation and/or other materials
;                provided with the distribution.
;
;           THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER AND CONTRIBUTORS
;           "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
;           LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
;           FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
;           COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
;           INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
;           (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
;           SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
;           HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
;           STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
;           ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
;           OF THE POSSIBILITY OF SUCH DAMAGE.
;
;   DEUTSCH:
;           Copyright 2006-2017 Karl Loncarek
;           Alle Rechte vorbehalten.
;
;           Weiterverbreitung in kompilierter oder nichtkompilierter Form, mit
;           oder ohne Vernderung, sind unter den folgenden Bedingungen
;           zulssig:
;
;             1. Weiterverbreitete nichtkompilierte Exemplare mssen das obere
;                Copyright, die Liste der Bedingungen und den folgenden
;                Verzicht im Sourcecode enthalten.
;             2. Weiterverbreitete kompilierte Exemplare mssen das obere
;                Copyright, die Liste der Bedingungen und den folgenden
;                Verzicht in der Dokumentation und/oder anderen Materialien,
;                die mit dem Exemplar verbreitet werden, enthalten.
;
;           DIESE SOFTWARE WIRD VOM INHABER DES COPYRIGHTS UND DEN BEITRAGENDEN
;           OHNE JEGLICHE SPEZIELLE ODER IMPLIZIERTE GARANTIEN ZUR VERFGUNG
;           GESTELLT, DIE UNTER ANDEREM EINSCHLIESSEN: DIE IMPLIZIERTE GARANTIE
;           DER VERWENDBARKEIT DER SOFTWARE FR EINEN BESTIMMTEN ZWECK. AUF
;           KEINEN FALL SIND DER INHABER DES COPYRIGHTS ODER DIE BEITRAGENDEN
;           FR IRGENDWELCHE DIREKTEN, INDIREKTEN, ZUFLLIGEN, SPEZIELLEN,
;           BEISPIELHAFTEN ODER FOLGENDEN SCHDEN (UNTER ANDEREM VERSCHAFFEN
;           VON ERSATZGTERN ODER -DIENSTLEISTUNGEN; EINSCHRNKUNG DER
;           NUTZUNGSFHIGKEIT; VERLUST VON NUTZUNGSFHIGKEIT; DATEN; PROFIT
;           ODER GESCHFTSUNTERBRECHUNG), WIE AUCH IMMER VERURSACHT UND UNTER
;           WELCHER VERPFLICHTUNG AUCH IMMER, OB IN VERTRAG, STRIKTER
;           VERPFLICHTUNG ODER UNERLAUBTE HANDLUNG (INKLUSIVE FAHRLSSIGKEIT)
;           VERANTWORTLICH, AUS WELCHEM WEG SIE AUCH IMMER DURCH DIE BENUTZUNG
;           DIESER SOFTWARE ENTSTANDEN SIND, SOGAR, WENN SIE AUF DIE
;}           MGLICHKEIT EINES SOLCHEN SCHADENS HINGEWIESEN WORDEN SIND.
;
; Contributors / Beitragende:
;   Karl Loncarek - karl@loncarek.de - aka Klonk
;                   see: http://www.loncarek.de (german)
;
;{ History: (+ new; ! bugfix; * improvement/change ) / Historie: (+ neu; !Bugfix; * Verbesserung)
;   1.0   - 02.04.2006 +first public release
;   1.0.1 - 03.04.2006 *optimized speed of displaying the tree in the
;                       configuration window; save only when changes
;                      *added checking for network drive in absolute
;                       path creation
;   1.0.2 - 04.04.2006 *changed positions of requesters to be more pleasant
;   1.1   - 04.04.2006 +environment variables within the commandline, options
;                       or homedir are replaced
;   1.2   - 05.04.2006 +optionally started applications are closed on exit
;                       of uStart
;                      *improved display of included help
;   1.2.1 - 05.04.2006 !closing of applications fixed; when other program is
;                       started after application launch, closing did not work
;   1.2.2 - 07.04.2006 !when an application started with uStart was closed
;                       manually, Start hung
;   1.2.3 - 09.04.2006 !again the automatic closing: fixed strange behaviour,
;                       sometimes was not able to close a program when its
;                       window was maximized minimized several times
;   1.2.4 - 13.04.2006 !minor bugfixes
;                      !sometimes launching an application took very long
;                      *count check when starting applications, avoids hangs
;   1.3   - 21.04.2006 +added support for other files than .exe
;                      *added asking for save on exiting when actual application
;                       entry is changed
;                      +added support for folders (opened through explorer)
;                      +added possibility to run applications automatically on 
;                       program start
;                      !minor bugfixes
;   1.4   - 02.05.2006 *now better question when exiting setting of applications
;                      *question on exiting configure window (added cancel)
;                      *changed positioning of messagerequester, created own one
;                      *improved handling of entries at the end of the list
;                      !major rewrite: "move up" and "move down", did not work
;                       correctly with more complex menu structures
;                      !delete function in configuring applications
;                      !minor bugfixes
;                      +added BSD-style license in german and english language
;   1.4.1 - 13.07.2006 !changed displayed title from "Start" to "uStart" to
;                       improve compatibilty with double-byte languages
;                      *added support for "\" as relative path, meaning the root
;                       directory
;                      *optimized creation of relative paths
;                      +added question for directly jumping to definition of
;                       applications when none were set yet
;   1.5   - 06.09.2006 *added option to define which applications should be
;                       automatically closed; works around problem with some
;                       applications that might hang on start
;                      !now all programs should end on exit
;                      !minor bugfixes
;   1.5.1 - 27.02.2007 *Close on exit button is now only available, when an exe
;                       is inserted
;                      *improved again automatic closing of applications, now
;                       it is waited longer before the application has finally
;                       started (up to 40secs in total). A message is also shown.
;                      !fixed special message requester to support CR & LF
;   1.5.2 - 10.04.2007 !added confirmation when saving entry
;   1.5.3 - 31.03.2014 * Updated for PureBasic 5.22
;                      !fixed a bug when moving down and last entry is a submenu,
;                       it was moved into the followig submenu
;                      !fixed wrong movement of menuitems i.e. display of menu 
;                       is incorrect
;   1.5.4 - 11.11.2015 !fixed problem, when closing groups in traymenu generation
;   1.6.0 - 24.07.2017 *Updated for Purebasic 5.60
;                      +Added some menuicons
;                      +Icons are now displayed in the tray menu (same as shown
;                       in windows explorer)
;                      !Hopefully now automatic closing of startd applications 
;                       works now correctly
;                      *Configuratin menu now centered in screen
;                      !minor bugfixes
;                      *minor changes in wording
;}
; **************************************************************************
#AppName = "uStart v1.6.0"
#PrefFile = "uStart.prefs"
;{  in ustart.prefs file the following types are available:
;   type = 1      Application
;   type = 2      Group
;   type = 3      Separator
;   type = 4      Folder
;}

Structure App
  type.b      ;Type of entry: 1:Application, 2:Group, 3:Separator, 4:Folder
  run.b       ;Flag whether this application should run automatically on start
  stop.b      ;Flag whether this application should end automatically on exit
  level.b     ;Level in treegadget, means separator
  name.s      ;Application name, empty => group
  command.s   ;Command line to call the program, empty => group
  options.s   ;Command line options
  homedir.s   ;Homedirectory
  hWnd.i      ;window handle of the started program
  hIcon.i     ;Handle of the file icon
EndStructure
NewList LaunchApp.App()

Global Window_0, Window_1, ProgramPath$, DriveLetter$, IsChanged.b = 0, Tree_0

Enumeration
  #TrayMenu
  #AppMenu
  #String_0
  #String_1
  #String_2
  #String_3
  #Text_0
  #Text_1
  #Text_2
  #Text_3
  #Text_4
  #Text_6
  #Frame3D_0
  #Button_0
  #Button_1
  #Button_2
  #Button_3
  #Button_4
  #Button_5
  #Button_6
  #Button_7
  #Button_8
  #Button_9
  #Button_10
  #Radio_0
  #Radio_1
  #Radio_2
  #Radio_3
  #Editor_1
  #Check_0
  #Check_1
EndEnumeration

;{ ==========================================================================
;|  MessageRequesterEx(title$,text$,flags,xPos=-1,yPos=-1)
;|    This is a special procedure to display a window in the same way as a
;|    MessageRequester. When yPos is not provided then xPos will be treated
;|    as winID. winID is the value provided by an OpenWindow. If winID is
;|    valid the Requester will be displayed centered to that window. If the
;|    winID is not valid then it is possible to set the position of the top
;|    left edge of the requester window manually xPos and yPos are only used,
;|    when no winID was given. When no windowID or xPos or yPos is given,
;|    then the message is displayed cenered to the screen. The same constants
;|    are used than with MessageRequester. No OS specific flags (e.g. from
;|    WinAPI) are supported!
;|    If the flag is set to a value of 0 then no button is displayed and the
;|    windowID is returned so the window can be closed with CloseWindow
;} ==========================================================================
Procedure MessageRequesterEx(title$,text$,flags=-2,xPos=-1,yPos=-1)
;***** display message requester at special position or centered to parent window
  Structure Button
    text.s  ;contains Buttontext
    id.l    ;contains ButtonID
    val.l   ;contains Button Returnvalue
  EndStructure
  Dim Buttons.Button(3)  ;maximum of 3 buttons possible
  #buttonwidth = 75 ;set width per button (pixels)
  #buttonspace = 6 ;set space between buttons (pixels)
  #lineheight = 13 ;set height per line (pixels)
  #minheight = 75 ;set minimum height of the requester window for one line text
  #widthcorrection = 0.78 ; correction factor for textwidth calculation
  ExamineDesktops() ; allow checking for desktopsize
  If flags = -2 ;set default if no flags provided
    flags = #PB_MessageRequester_Ok
  EndIf ;set default for WinID
  If yPos = -1
    winID = xPos
  Else
    winID = -1
  EndIf
  ;{ calculate necessary number of buttons and set texts for buttons
  If flags = -1
    ButtonCount = 0
  ElseIf flags = #PB_MessageRequester_Ok
    ButtonCount = 1
    Buttons(1)\text = "Ok"
    Buttons(1)\val = #PB_MessageRequester_Ok
  ElseIf flags = #PB_MessageRequester_YesNo
    ButtonCount = 2
    Buttons(1)\text = "Yes"
    Buttons(1)\val = #PB_MessageRequester_Yes
    Buttons(2)\text = "No"
    Buttons(2)\val = #PB_MessageRequester_No
  ElseIf flags = #PB_MessageRequester_YesNoCancel
    ButtonCount = 3
    Buttons(1)\text = "Yes"
    Buttons(1)\val = #PB_MessageRequester_Yes
    Buttons(2)\text = "No"
    Buttons(2)\val = #PB_MessageRequester_No
    Buttons(3)\text = "Cancel"
    Buttons(3)\val = #PB_MessageRequester_Cancel
  EndIf
  ;}
  ;{ calculate maximum allowed width of requester
  maximumwidth = Round(DesktopWidth(0)*0.62,0)
  If (xPos<>-1) And (yPos<>-1) And ((DesktopWidth(0)-xPos)<maximumwidth) ;make requester not bigger than available space
    maximumwidth = DesktopWidth(0)-xPos
  EndIf
  ;}
  ;{ calculate necessary width And height of the requester
  image = CreateImage(#PB_Any,8,8) ;create temporary image for checking of textsize
  StartDrawing(ImageOutput(Image))
  textlines = 1 ;minimum one line
  text$ = ReplaceString(text$,Chr(13)+Chr(10),Chr(13)) ;Replace CR+LF by only CR
  textlines = 0 ;count needed lines of text, set default value
  RequesterWidth = 0
  temp$ = text$
Debug(TextWidth(temp$))
  For i = 1 To Len(temp$)
    If (TextWidth(Left(temp$,i))+20 >= maximumwidth) Or (Right(Left(temp$,i),1)=Chr(13)) Or (Right(Left(temp$,i),1)=Chr(10))
      If (Right(Left(temp$,i),1)=Chr(13)) Or (Right(Left(temp$,i),1)=Chr(10)) ;new line because of CR of LF
        If ((TextWidth(Left(temp$,i-1))+20) > RequesterWidth)
          RequesterWidth = (TextWidth(Left(temp$,i-1))+20)
        EndIf
      EndIf
      textlines=textlines+1
      temp$ = Right(temp$,Len(temp$)-i)
      i=1
    Else
      If ((TextWidth(Left(temp$,i))+20) > RequesterWidth)
        RequesterWidth = (TextWidth(Left(temp$,i))+20)
      EndIf
    EndIf
  Next
  RequesterHeight = #minheight+(textlines)*#lineheight ;calculate height of messagerequester
  If RequesterWidth < (ButtonCount*#buttonwidth+(ButtonCount-1)*#buttonspace)
    RequesterWidth = (ButtonCount*#buttonwidth+(ButtonCount-1)*#buttonspace)+20
  EndIf
  If (ButtonCount = 0) ;correct height if no button is displayed
    RequesterHeight = RequesterHeight - 35
  EndIf
  StopDrawing()
  FreeImage(image) ;free memory for temporary image
  ;} 
  ;{ open window according to provided data
  If IsWindow(winID) 
    Requester = OpenWindow(#PB_Any, 0, 0, RequesterWidth, RequesterHeight, title$, #PB_Window_WindowCentered, WindowID(winID))
  ElseIf (xPos<>-1) And (yPos<>-1)
    Requester = OpenWindow(#PB_Any, xPos, yPos, RequesterWidth, RequesterHeight, title$, 0)
  Else
    Requester = OpenWindow(#PB_Any, 0, 0, RequesterWidth, RequesterHeight, title$, #PB_Window_ScreenCentered)
  EndIf
  ;}
  If Requester
    TextGadget(#PB_Any, 13,11, RequesterWidth-10, (textlines+1)*#lineheight+5, text$)
    If (ButtonCount <> 0)
      For i = 1 To ButtonCount
        Buttons(i)\id = ButtonGadget(#PB_Any, Round((Requesterwidth-(ButtonCount*#buttonwidth)-((ButtonCount-1)*#buttonspace))/2,0)+((i-1)*(#buttonwidth+#buttonspace)),Requesterheight-33 ,#buttonwidth,23, Buttons(i)\text)
      Next
    EndIf
  EndIf
  If (ButtonCount <> 0)
    Repeat ;handle event for message requester window
      Event = WaitWindowEvent()
      If Event = #PB_Event_Gadget ;button was pressed
        For i=1 To ButtonCount
          If EventGadget() = Buttons(i)\id ;determine which button was pressed
            CloseWindow(Requester) ;close Requesterwindow
            ProcedureReturn Buttons(i)\val ;give back result
          EndIf
        Next
      EndIf
    ForEver
  Else
    ProcedureReturn Requester
  EndIf
EndProcedure

Procedure Open_Window_0()
  Window_0 = OpenWindow(#PB_Any, 0, 0, 506, 217, "Help/About - "+#AppName, #PB_Window_SystemMenu | #PB_Window_Invisible | #PB_Window_ScreenCentered)
  If Window_0
    ;Add icon to systray
    AddSysTrayIcon(1, WindowID(Window_0), ImageID(CatchImage(#PB_Any, ?Trayicon)))
    SysTrayIconToolTip(1, "uStart"+Chr(13)+"Left-Click -> Launch Apps"+Chr(13)+"Right-Click -> Help/About/Exit")
    ;create Trayicon menu
    If CreatePopupImageMenu(#TrayMenu)
      MenuItem(4, "Activate automatic END on exit")
      MenuItem(5, "Activate automatic RUN on start")
      MenuBar()
      MenuItem(3, "Set Applications", ExtractIcon_(GetModuleHandle_(0),"shell32.dll",69))
      MenuItem(2, "Help/About", ExtractIcon_(GetModuleHandle_(0),"shell32.dll",23))
;      MenuItem(3, "Set Applications", ImageID(CatchImage(#PB_Any, ?Prefs)))
;      MenuItem(2, "Help/About", ImageID(CatchImage(#PB_Any, ?Help)))
      MenuBar()
;      MenuItem(1,"Exit", ImageID(CatchImage(#PB_Any, ?Exit)))
      MenuItem(1,"Exit", ExtractIcon_(GetModuleHandle_(0),"shell32.dll",27))
      SetMenuItemState(#TrayMenu,4,1)
      SetMenuItemState(#TrayMenu,5,1)
    EndIf
    If CreatePopupImageMenu(#AppMenu)
      MenuItem(10, "(empty)")
      DisableMenuItem(#AppMenu,10,1)
    EndIf
    Restore Helptext
    Read.b linecount.b
    For i=1 To linecount
      Read.s tmp$
      text$ = text$ + tmp$ + Chr(13) + Chr(10)
    Next i
    EditorGadget(#Editor_1, 5, 5, 496, 180, #PB_Editor_ReadOnly)
    SetGadgetFont(#Editor_1, LoadFont(0, "Arial", 10))
    SetGadgetText(#Editor_1, text$)
    ButtonGadget(#Button_10, 213, 190, 80, 20, "OK")
  EndIf
EndProcedure

Procedure Open_Window_1()
  ;***** Configuration Window
  Window_1 = OpenWindow(#PB_Any, 0, 0, 506, 217, "Configure - "+#AppName, #PB_Window_ScreenCentered)
  If Window_1
    TextGadget(#Text_0, 5, 0, 160, 15, "Current application launch menu:")
    SetGadgetFont(#Text_6, FontID(LoadFont(#PB_Any, "Arial", 7, #PB_Font_Italic)))
    Tree_0 = TreeGadget(#PB_Any, 0, 15, 160, 200, #PB_Tree_AlwaysShowSelection)
    FrameGadget(#Frame3D_0, 250, 0, 255, 35, "Entry type")
    OptionGadget(#Radio_0, 255, 15, 75, 15, "Application")
    SetGadgetState(#Radio_0,1)
    GadgetToolTip(#Radio_0, "Application to run or file to open")
    OptionGadget(#Radio_3, 330, 15, 50, 15, "Folder")
    GadgetToolTip(#Radio_3, "Folder to open")
    OptionGadget(#Radio_1, 380, 15, 50, 15, "Group")
    GadgetToolTip(#Radio_1, "Group in application menu")
    OptionGadget(#Radio_2, 430, 15, 65, 15, "Separator")
    GadgetToolTip(#Radio_2, "Separator in application menu")
    ButtonGadget(#Button_0, 165, 15, 80, 20, "Insert")
    DisableGadget(#Button_0, 1)
    GadgetToolTip(#Button_0, "Insert before current selection")
    ButtonGadget(#Button_1, 165, 35, 80, 20, "Add")
    DisableGadget(#Button_1, 1)
    GadgetToolTip(#Button_1, "Add after current selection")
    ButtonGadget(#Button_2, 165, 60, 80, 20, "Save Change")
    DisableGadget(#Button_2, 1)
    ButtonGadget(#Button_9, 165, 80, 80, 20, "Restore/Clear")
    DisableGadget(#Button_9, 1)
    GadgetToolTip(#Button_2, "Save changes to current selection")
    ButtonGadget(#Button_3, 165, 105, 80, 20, "Delete")
    DisableGadget(#Button_3, 1)
    GadgetToolTip(#Button_3, "Delete current selection")
    ButtonGadget(#Button_4, 165, 130, 80, 20, "Move Up")
    GadgetToolTip(#Button_4, "Move up selected entry")
    DisableGadget(#Button_4, 1)
    ButtonGadget(#Button_5, 165, 150, 80, 20, "Move Down")
    GadgetToolTip(#Button_5, "Move down selected entry")
    DisableGadget(#Button_5, 1)
    ButtonGadget(#Button_6, 165, 175, 80, 20, "Exit")
    TextGadget(#Text_1, 255, 40, 155, 15, "Name:")
    StringGadget(#String_0, 250, 55, 255, 20, "")
    GadgetToolTip(#String_0, "This is displayed in the Launch Apps menu")
    TextGadget(#Text_2, 255, 80, 155, 15, "Application / Filepath:")
    StringGadget(#String_1, 250, 95, 235, 20, "")
    GadgetToolTip(#String_1, "Path and filename of the executable (relative path)")
    ButtonGadget(#Button_7, 485, 95, 20, 20, "...")
    TextGadget(#Text_3, 255, 120, 235, 15, "Command line parameters (%drive% -> e.g. G:):")
    StringGadget(#String_2, 250, 135, 255, 20, "")
    GadgetToolTip(#String_2, "%drive% is replaced by the current drive letter, e.g. G:")
    TextGadget(#Text_4, 255, 160, 155, 15, "Working directory:")
    StringGadget(#String_3, 250, 175, 235, 20, "")
    GadgetToolTip(#String_3, "Only necessary for some applications (realtive path)")
    ButtonGadget(#Button_8, 485, 175, 20, 20, "...")
    CheckBoxGadget(#Check_0, 250, 200, 110, 15, "RUN on Start")
    GadgetToolTip(#Check_0, "Runs this application when starting uStart and tray option is active")
    CheckBoxGadget(#Check_1, 360, 200, 100, 15, "END on Exit")
    GadgetToolTip(#Check_1, "Ends this application on exit of uStart and tray option is active")
  EndIf
EndProcedure

Procedure UpdateTrayMenu()
;*****create the left-click tray menu according to settings
  Shared LaunchApp()
  SelectedItem = ListIndex(LaunchApp()) ;save actual position in list
  If ListSize(LaunchApp())>0
    FreeMenu(#AppMenu)
    If CreatePopupImageMenu(#AppMenu)  ;Create TrayMenu
      LastLevel = -1
      
      ForEach LaunchApp()
        If LaunchApp()\level < LastLevel
          For i = 1 To Lastlevel - LaunchApp()\level ;close menues across several levels
            CloseSubMenu()
          Next i 
          LastLevel = LaunchApp()\level ; restore only down to current level
        EndIf
        If (LaunchApp()\type = 1) Or (LaunchApp()\type = 4);application or folder
          MenuItem(10+ListIndex(LaunchApp()),LaunchApp()\name,LaunchApp()\hIcon)
        ElseIf LaunchApp()\type = 3 ;separator
          MenuBar()
        ElseIf LaunchApp()\type = 2 ;group
          OpenSubMenu(LaunchApp()\name)
          LastLevel = LaunchApp()\level + 1
        EndIf
      Next
      
      SelectElement(LaunchApp(),SelectedItem) ;restore position in list
    EndIf
  ElseIf GetMenuItemText(#AppMenu,10)<>"(empty)"
    FreeMenu(#AppMenu)
    If CreatePopupMenu(#AppMenu)
      MenuItem(10, "(empty)")
      DisableMenuItem(#AppMenu,10,1)
    EndIf    
  EndIf
EndProcedure

;{ ==========================================================================
;|  GetFileIcon(File$)
;|    Uses WinAPI function to obtain the icon of File$. Works also for 
;|    non-exe, non-dll etc. That Icon is obtained, that is shown by windows.
;|    Result is Handle to Icon image. Don't forget to free the Memory with 
;|    "DestroyIcon_" WinAPI function If not needed anymore (e.g. on exit)!
;|    Icon sizes: #SHGFI_SMALLICON 16x16, #SHGFI_LARGEICON 32x32
;} ==========================================================================
Procedure GetFileIcon(File$)
  Protected SHFILEINFO.SHFILEINFO
  Protected hIcon
  If SHGetFileInfo_(@File$, #FILE_ATTRIBUTE_NORMAL, @SHFILEINFO, SizeOf(SHFILEINFO), #SHGFI_SMALLICON | #SHGFI_ICON)
    hIcon = SHFILEINFO\hIcon ;obtain icon handle
    ProcedureReturn hIcon
  EndIf
EndProcedure

;{ ==========================================================================
;|  ConvertToRelativePath(TargetPath$,ActualPath$)
;|    Returns the path of TargetPath$ in relation to ActualPath$, i.e creates
;|    a relative path to TargetPath$ with ActualPath$ as starting point. When
;|    the directory is identical the result will be "./". When TargetPath$ is
;|    located on a different drive TargetPath$ is given back.
;} ==========================================================================
Procedure.s ConvertToRelativePath(TargetPath$,ActualPath$)
  ;***** TargetPath$ = Path to be converted (relative to ActualPath$)
  If Left(TargetPath$,3)<>Left(ActualPath$,3)
    ;Target is located on a different drive -> no relative path possible
    RelativePath$ = TargetPath$
  ElseIf Left(TargetPath$,FindString(TargetPath$,"\",4)) <> Left(ActualPath$,FindString(ActualPath$,"\",4))
    ;Target is only located on the same drive, everything is different -> Path could start with "\"
    RelativePath$ = Right(TargetPath$,Len(TargetPath$)-2)
  ElseIf TargetPath$<>ActualPath$
    ;remove identical parts of both paths
    backslashposition = 0
    Repeat
      TargetPath$ = Right(TargetPath$,Len(TargetPath$)-backslashposition)
      ActualPath$ = Right(ActualPath$,Len(ActualPath$)-backslashposition)
      backslashposition = FindString(TargetPath$,"\",1)
    Until Left(TargetPath$,backslashposition) <> Left(ActualPath$,FindString(ActualPath$,"\",1))
    ;get number of remaining backslashes
    backslashcount = CountString(ActualPath$,"\")
    RelativePath$ = TargetPath$
    ;add relative paths
    For i=1 To backslashcount
      RelativePath$ = "..\" + RelativePath$
    Next i
  Else
    ;Target is located in the same directory
    RelativePath$ =  ".\"
  EndIf
  ProcedureReturn RelativePath$
EndProcedure

;{ ==========================================================================
;|  ConvertToAbsolutePath(TargetPath$,ActualPath$)
;|    Returns the absolute path (with driveletter etc.) to the relative
;|    TargetPath$ where ActualPath$ is the starting point. When ".\" is given
;|    then the result will be ActualPath$. When "" is given the result will
;|    be empty also
;} ==========================================================================
Procedure.s ConvertToAbsolutePath(TargetPath$,ActualPath$)
  ;***** TargetPath$ = relative to be converted in relation to ActualPath$)
  If TargetPath$ = ".\" ;same as ActualPath
    AbsolutePath$ = ActualPath$
  ElseIf (Mid(TargetPath$, 2, 1) = ":") Or (Left(TargetPath$,2) = "\\") Or (Left(TargetPath$,1) = "%")
    ;Targetpath has a drive letter or is network drive or starts with environment variable
    AbsolutePath$ = TargetPath$
  ElseIf (Left(TargetPath$,1) = "\")
    ;Targetpath starts with "\" -> on same drive
    AbsolutePath$ = Left(GetPathPart(StartingPath$),2) + TargetPath$
  Else ;some other relative path
    ActualPath$ = Left(ActualPath$,Len(ActualPath$)-1) ;remove last backslash
    While Left(TargetPath$,3) = "..\"
      TargetPath$ = Right(TargetPath$, Len(TargetPath$)-3)
      lastbackslashposition = 1
      While FindString(ActualPath$,"\",lastbackslashposition+1)<>0
        lastbackslashposition = FindString(ActualPath$,"\",lastbackslashposition+1)
      Wend
      ActualPath$ = Left(ActualPath$,lastbackslashposition-1) ;remove unecessary part
    Wend
    AbsolutePath$ = ActualPath$ + "\" + TargetPath$
  EndIf
  ProcedureReturn AbsolutePath$
EndProcedure

Procedure LoadData()
;*****Loads saved application settings
  Shared LaunchApp()
  ClearList(LaunchApp())
  If FileSize(ProgramPath$+#PrefFile)<>-1 Or FileSize(ProgramPath$+#PrefFile+".bak")<>-1
    If FileSize(ProgramPath$+#PrefFile)=-1 ; file not found
      RenameFile(ProgramPath$+#PrefFile+".bak",ProgramPath$+#PrefFile)
    EndIf
    OpenPreferences(ProgramPath$+#PrefFile)
    PreferenceGroup("Settings")
    SetMenuItemState(#TrayMenu,4,ReadPreferenceLong("EndOnExit",0))
    SetMenuItemState(#TrayMenu,5,ReadPreferenceLong("RunOnStart",1))
    ExaminePreferenceGroups()
    While (NextPreferenceGroup()<>0)
      If Left(PreferenceGroupName(),4)="Item"
        AddElement(LaunchApp())
        LaunchApp()\type = Val(ReadPreferenceString("type","0"))
        LaunchApp()\level = Val(ReadPreferenceString("level","0"))
        ; reset values
        LaunchApp()\name=""
        LaunchApp()\command=""
        LaunchApp()\options=""
        LaunchApp()\homedir=""
        LaunchApp()\run=0
        LaunchApp()\stop=0
        LaunchApp()\hIcon=0
        ;fill values into list
        If LaunchApp()\type = 1 ;Application
          LaunchApp()\run = Val(ReadPreferenceString("run","0"))
          LaunchApp()\stop = Val(ReadPreferenceString("stop","0"))
          LaunchApp()\name = ReadPreferenceString("name","")
          LaunchApp()\command = ReadPreferenceString("command","")
          LaunchApp()\options = ReadPreferenceString("options","")
          LaunchApp()\homedir = ReadPreferenceString("homedir","")
          LaunchApp()\hIcon = GetFileIcon(ConvertToAbsolutePath(LaunchApp()\command,ProgramPath$))
        ElseIf LaunchApp()\type = 2 ;Group
          LaunchApp()\name = ReadPreferenceString("name","")
        ElseIf LaunchApp()\type = 4 ;Folder
          LaunchApp()\run = Val(ReadPreferenceString("run","0"))
          LaunchApp()\stop = Val(ReadPreferenceString("stop","0"))
          LaunchApp()\name = ReadPreferenceString("name","")
          LaunchApp()\homedir = ReadPreferenceString("homedir","")
        EndIf
      EndIf
    Wend
    ClosePreferences()
    UpdateTrayMenu()
  EndIf
EndProcedure

Procedure SaveData()
;*****Saves application settings
  Shared LaunchApp()
  SelectedItem = ListIndex(LaunchApp()) ; save actual position in list
  If FileSize(ProgramPath$+#PrefFile)<>-1
    RenameFile(ProgramPath$+#PrefFile,ProgramPath$+#PrefFile+".bak")
  EndIf
  If CreatePreferences(ProgramPath$+#PrefFile)=0
    MessageRequesterEx("Write Error","Settings could not be saved!",#PB_MessageRequester_Ok, Window_1)
  Else
    PreferenceGroup("Settings")
    WritePreferenceLong("EndOnExit",GetMenuItemState(#TrayMenu,4))
    WritePreferenceLong("RunOnStart",GetMenuItemState(#TrayMenu,5))
    ForEach LaunchApp()
      PreferenceGroup("Item"+Str(ListIndex(LaunchApp())))
      WritePreferenceString("type",Str(LaunchApp()\type))
      WritePreferenceString("level", Str(LaunchApp()\level))
      If LaunchApp()\type = 1 ;Application
        WritePreferenceString("run",Str(LaunchApp()\run))
        WritePreferenceString("stop",Str(LaunchApp()\stop))
        WritePreferenceString("name",LaunchApp()\name)
        WritePreferenceString("command",LaunchApp()\command)
        WritePreferenceString("options",LaunchApp()\options)
        WritePreferenceString("homedir",LaunchApp()\homedir)
      ElseIf LaunchApp()\type = 2 ;Group
        WritePreferenceString("name",LaunchApp()\name)
      ElseIf LaunchApp()\type = 4 ;Folder
        WritePreferenceString("run",Str(LaunchApp()\run))
        WritePreferenceString("stop",Str(LaunchApp()\stop))
        WritePreferenceString("name",LaunchApp()\name)
        WritePreferenceString("homedir",LaunchApp()\homedir)
      EndIf
    Next
    ClosePreferences()
    DeleteFile(ProgramPath$+#PrefFile+".bak")
    If ListIndex(LaunchApp())<>-1
      SelectElement(LaunchApp(),SelectedItem) ;restore position in list
    EndIf
  EndIf
EndProcedure

Procedure StoreData()
;*****Saves the actual settings in the list (except the level)
  Shared LaunchApp()
  If GetGadgetState(#Radio_0) = 1
    ;Application
    LaunchApp()\type = 1
    LaunchApp()\run = GetGadgetState(#Check_0)
    LaunchApp()\stop = GetGadgetState(#Check_1)
    LaunchApp()\name = Trim(GetGadgetText(#String_0))
    LaunchApp()\command = Trim(GetGadgetText(#String_1))
    LaunchApp()\options = Trim(GetGadgetText(#String_2))
    LaunchApp()\homedir = Trim(GetGadgetText(#String_3))
    If LaunchApp()\homedir = ""
      LaunchApp()\homedir = GetPathPart(LaunchApp()\command)
    EndIf
    DestroyIcon_(LaunchApp()\hIcon) ;free memory used for icons
    LaunchApp()\hIcon = GetFileIcon(LaunchApp()\command)
  ElseIf GetGadgetState(#Radio_1) = 1
    ;Group
    LaunchApp()\type = 2
    LaunchApp()\name = Trim(GetGadgetText(#String_0))
    LaunchApp()\command = ""
    LaunchApp()\options = ""
    LaunchApp()\homedir = ""
  ElseIf GetGadgetState(#Radio_2) = 1
    ;Separator
    LaunchApp()\type = 3
    LaunchApp()\name = ""
    LaunchApp()\command = ""            
    LaunchApp()\options = ""
    LaunchApp()\homedir = ""
  ElseIf GetGadgetState(#Radio_3) = 1
    ;Folder
    LaunchApp()\type = 4
    LaunchApp()\run = GetGadgetState(#Check_0)
    LaunchApp()\stop = GetGadgetState(#Check_1)
    LaunchApp()\name = Trim(GetGadgetText(#String_0))
    LaunchApp()\command = ""            
    LaunchApp()\options = ""
    LaunchApp()\homedir = Trim(GetGadgetText(#String_3))
  EndIf
EndProcedure

Procedure ExpandTree(Actual)
;*****Expands the displayed tree according to selected item
  SetGadgetItemState(tree_0,Actual, #PB_Tree_Selected | #PB_Tree_Expanded)
  actuallevel = GetGadgetItemAttribute(Tree_0,Actual,#PB_Tree_SubLevel) ; save level of item
  If (Actual>0) And (actuallevel>0) ;open all necessary trees to show selected item
    nextlevel = actuallevel
    Repeat
      Actual = Actual -1
      level = GetGadgetItemAttribute(Tree_0,Actual,#PB_Tree_SubLevel)
      If nextlevel > level
        SetGadgetItemState(Tree_0,Actual,#PB_Tree_Expanded)
        nextlevel = level
      EndIf
    Until level = 0
  EndIf
EndProcedure

Procedure UpdateValueDisplay(SelectedItem)
;****Shows the values of the actual selected items
  Shared LaunchApp()
  If ListIndex(LaunchApp())<>-1
    SelectElement(LaunchApp(),SelectedItem)
    SetGadgetText(#String_0,LaunchApp()\name)
    SetGadgetText(#String_1,LaunchApp()\command)
    SetGadgetText(#String_2,LaunchApp()\options)
    SetGadgetText(#String_3,LaunchApp()\homedir)
    SetGadgetState(#Check_0,LaunchApp()\run)
    SetGadgetState(#Check_1,LaunchApp()\stop)
    If LaunchApp()\type = 1
      ;Application
      SetGadgetState(#Radio_0,1)
      DisableGadget(#String_0,0)
      DisableGadget(#String_1,0)
      DisableGadget(#String_2,0)
      DisableGadget(#String_3,0)
      DisableGadget(#Button_7,0)
      DisableGadget(#Button_8,0)
    ElseIf LaunchApp()\type = 2
      ;Group              
      SetGadgetState(#Radio_1,1)
      DisableGadget(#String_0,0)
      DisableGadget(#String_1,1)
      DisableGadget(#String_2,1)
      DisableGadget(#String_3,1)
      DisableGadget(#Button_7,1)
      DisableGadget(#Button_8,1)
    ElseIf LaunchApp()\type = 3
      ;Separator
      SetGadgetState(#Radio_2,1)
      DisableGadget(#String_0,1)
      DisableGadget(#String_1,1)
      DisableGadget(#String_2,1)
      DisableGadget(#String_3,1)
      DisableGadget(#Button_7,1)
      DisableGadget(#Button_8,1)
    ElseIf LaunchApp()\type = 4
      ;folder
      SetGadgetState(#Radio_3,1)
      DisableGadget(#String_0,0)
      DisableGadget(#String_1,1)
      DisableGadget(#String_2,1)
      DisableGadget(#String_3,0)
      DisableGadget(#Button_7,1)
      DisableGadget(#Button_8,0)
    EndIf
  EndIf
EndProcedure

Procedure UpdateTreeDisplay(SelectedItem)
;*****Updates the display within the treegadget
  Shared LaunchApp()
  ClearGadgetItems(tree_0)
  ResetList(LaunchApp())  
  While NextElement(LaunchApp())
    ;process all element in the list
    If LaunchApp()\name <> "" ; group or application or folder
      AddGadgetItem(Tree_0, -1, LaunchApp()\name, 0, LaunchApp()\level)
    Else
      AddGadgetItem(Tree_0, -1, "--------------", 0, LaunchApp()\level)
    EndIf
  Wend
  SetGadgetState(Tree_0,SelectedItem) ;Set selection according to display
  ExpandTree(SelectedItem) ;expand trees when necessary
  UpdateValueDisplay(SelectedItem) ;display values
  UpdateTrayMenu()
EndProcedure

Procedure CheckEntries()
;***** change/check available buttons
  Shared LaunchApp(), IsChanged
  IsChanged = 0
  SelectedItem = ListIndex(LaunchApp()) ;save actual position in list
  If IsWindow(Window_1)<>0
    If CountGadgetItems(Tree_0) = 0 ; disable restore and delete
      DisableGadget(#Button_3,1)
      DisableGadget(#Button_9,1)
    Else ; enable restore and delete
      DisableGadget(#Button_3,0)
      DisableGadget(#Button_9,0)
      If GetGadgetState(#Radio_0) = 1 ; get type of actual application and hide run button
        ;Application
        type = 1
        DisableGadget(#Check_0,0)
        DisableGadget(#Check_1,0)
      ElseIf GetGadgetState(#Radio_1) = 1
        ;Group
        type = 2
        DisableGadget(#Check_0,1)
        DisableGadget(#Check_1,1)
      ElseIf GetGadgetState(#Radio_2) = 1
        ;Separator
        type = 3
        DisableGadget(#Check_0,1)
        DisableGadget(#Check_1,1)
      ElseIf GetGadgetState(#Radio_3) = 1
        ;Folder
        type = 4
        DisableGadget(#Check_0,0)
        DisableGadget(#Check_1,1)
      EndIf
      If ListSize(LaunchApp()) > 0 ; Set Text on button to restore or reset
        If (LaunchApp()\type = type) And (LaunchApp()\name = GetGadgetText(#String_0)) And (LaunchApp()\command = GetGadgetText(#String_1)) And (LaunchApp()\options = GetGadgetText(#String_2)) And (LaunchApp()\homedir = GetGadgetText(#String_3)) And (LaunchApp()\run = GetGadgetState(#Check_0)) And (LaunchApp()\stop = GetGadgetState(#Check_1))
          SetGadgetText(#Button_9,"Clear/New")
          GadgetToolTip(#Button_9,"Clear all fields")
          DisableGadget(#Button_2,1)
          IsChanged = 0
        Else
          SetGadgetText(#Button_9,"Restore")      
          GadgetToolTip(#Button_9,"Restore to saved values")
          DisableGadget(#Button_2,0)
          DisableGadget(#Button_3,1)
          ActualLevel = LaunchApp()\level
          ; when anything changed check whether it is a group and hide save changes when there are subitems
          If (LaunchApp()\type=2) And (NextElement(LaunchApp())<>0) And (LaunchApp()\level > ActualLevel) And (GetGadgetState(#Radio_1)<>1)
            DisableGadget(#Button_2,1)
            SelectElement(LaunchApp(),SelectedItem) ;restore position in list
          EndIf
          IsChanged = 1
        EndIf
      EndIf
    EndIf
    ; Enable buttons only when text is entered (fields are not empty)
    If ((GetGadgetText(#String_0)<>"") And (GetGadgetText(#String_1)<>"") And (GetGadgetState(#Radio_0)=1)) Or ((GetGadgetText(#String_0)<>"") And (GetGadgetState(#Radio_1)=1))  Or (GetGadgetState(#Radio_2)=1) Or ((GetGadgetText(#String_0)<>"") And (GetGadgetText(#String_3)<>"") And (GetGadgetState(#Radio_3)=1))
      If CountGadgetItems(Tree_0)<>0
        DisableGadget(#Button_0,0)
      EndIf
      DisableGadget(#Button_1,0)
    Else
      DisableGadget(#Button_0,1)
      DisableGadget(#Button_1,1)
      DisableGadget(#Button_2,1) ;disable save
    EndIf
    ; Enable Move buttons
    If CountGadgetItems(Tree_0) > 1 ;active only when at least two entries are available
      If GetGadgetState(Tree_0) > 0 ;enable Move UP
        DisableGadget(#Button_4,0)
      Else
        DisableGadget(#Button_4,1)
      EndIf
      ActualLevel = LaunchApp()\level
      LastItem = ListIndex(LaunchApp())
      While NextElement(LaunchApp())<>0 ; search for last element with same level of current selection
        If LaunchApp()\level <= ActualLevel
          LastItem = ListIndex(LaunchApp())
        EndIf
      Wend
      SelectElement(LaunchApp(),SelectedItem) ;restore position in list
      If (GetGadgetState(Tree_0) <> -1) ;enable "Move down"
        If (GetGadgetState(Tree_0)<LastItem) Or ((GetGadgetState(Tree_0)=LastItem) And (ActualLevel>0))
          DisableGadget(#Button_5,0)
        Else
          DisableGadget(#Button_5,1)
        EndIf
      EndIf
    EndIf
    If UCase(GetExtensionPart(GetGadgetText(#String_1)))="EXE" ;Disable End on exit if application is not an EXE
      DisableGadget(#Check_1,0)
      If ListSize(LaunchApp()) > 0 ;Do only when Array already has entries
        SetGadgetState(#Check_1,LaunchApp()\stop)
      EndIf
    Else
      DisableGadget(#Check_1,1)
      SetGadgetState(#Check_1,#PB_Checkbox_Unchecked)
    EndIf
  EndIf
EndProcedure

;{ ==========================================================================
;|  ReplaceEnvVar(text$)
;|    Returns the string text$ where all strings pointing to environment
;|    variables, e.g.(%windir%) are replaced by their content
;} ==========================================================================
Procedure.s ReplaceEnvVar(text$)
;*****Checks text$ if it contains %sometext% and replaces it with the
;     content of the environment variable sometext
  count.b = CountString(text$,"%")
  result$ = text$
  If count > 0  
    For i.b = 1 To count ;go through each part of the string surrounded by "%"
      If FindString(text$,"%",1)<>0
        part$ = Left(text$,FindString(text$,"%",1)-1)
      EndIf
      text$ = Right(text$,Len(text$)-Len(part$)-1)
      If (i<>1) And (part$<>"") And (GetEnvironmentVariable(part$)<>"")
        ;replace found stuff by content of environment variable
        result$ = ReplaceString(result$,"%"+part$+"%",GetEnvironmentVariable(part$))
      EndIf
    Next
  EndIf
  ProcedureReturn result$
EndProcedure

;{ ==========================================================================
;|  RunProgramEx(filename$,parameter$,directory$)
;|    Same function as RunProgram except it uses WinAPI and gives back the 
;|    handle of the window. Thus this newly created window can be closed
;|    with PostMessage_(hWnd,#WM_CLOSE,0,0). Closing windows is only possible
;|    for EXE applications. It supports also starting i.e. opening of other
;|    files, e.g. .DOC with the associated applications.
;} ==========================================================================
Procedure RunProgramEx(filename$,parameter$,directory$)
  If Left(parameter$,1)<>" "
    parameter$=" "+parameter$
  EndIf
  Info.STARTUPINFO
  Info\cb=SizeOf(STARTUPINFO)
  Info\dwFlags=1
  Info\wShowWindow=#SW_SHOWNORMAL
  ProcessInfo.PROCESS_INFORMATION
  If UCase(GetExtensionPart(filename$))="EXE"
    If CreateProcess_(@filename$,@parameter$,0,0,0,#NORMAL_PRIORITY_CLASS ,0,@directory$,@Info,@ProcessInfo)
      ProcessID=ProcessInfo\dwProcessId
      Delay(100)
      WaitForInputIdle_(ProcessInfo\hProcess, 30000) ;wait for application to accept input, wait maximum 30 seconds
      count = 0
      Repeat
        Delay(100) ;avoid hanging of windows during start of application while searching for the window
        count=count + 1 ;security counter to avoid hanging
        win=FindWindow_(0,0)
        While win<>0
          GetWindowThreadProcessId_(win,@pid.l)
          If pid=ProcessID
            If GetParent_(win)=0 ;save handle only if not a child window
              WinHandle=win
              Break
            EndIf
          EndIf
          win=GetWindow_(win,#GW_HWNDNEXT) ;check next window
        Wend
      Until WinHandle Or (count=100) ;wait up to 10 seconds for window to occur
    EndIf
  Else
    RunProgram(filename$,parameter$,directory$)
    WinHandle = 0 ;just for security
  EndIf
  ProcedureReturn WinHandle
EndProcedure

Procedure RunApp()
;***** Runs the application according to the saved parameters
  Shared LaunchApp()
  prog$ = ReplaceEnvVar(ConvertToAbsolutePath(LaunchApp()\command,ProgramPath$))
  options$ = ReplaceEnvVar(ReplaceString(LaunchApp()\options,"%drive%",DriveLetter$))
  homedir$ = ReplaceEnvVar(ConvertToAbsolutePath(LaunchApp()\homedir,ProgramPath$))
  If LaunchApp()\type = 4 ;open folder with explorer
    RunProgram(ReplaceEnvVar("%windir%\explorer.exe"),homedir$,GetEnvironmentVariable("windir"))
  ElseIf (LaunchApp()\stop = 1) And (GetMenuItemState(#TrayMenu,4) = 1)
    message = MessageRequesterEx("Starting...","Application "+Chr(34)+LaunchApp()\name+Chr(34)+" is currently starting. Please be patient...",-1)
    LaunchApp()\hWnd = RunProgramEx(prog$,options$,homedir$) ;save Windowhandle
    CloseWindow(message)
  Else ;run program normally when no automatic close is selected
    LaunchApp()\hWnd = 0
    RunProgram(prog$,options$,homedir$)
  EndIf
EndProcedure

Procedure RunAppsAuto()
;***** Runs the selected applications on start
  Shared LaunchApp()
  If GetMenuItemState(#TrayMenu,5) = 1 ;only run applications when generally activated
    ForEach LaunchApp()
      If LaunchApp()\run = 1 ;run automatically is activated
        RunApp() ;run application
      EndIf
    Next
  EndIf
EndProcedure

ProgramPath$ = GetPathPart(ProgramFilename()) ;Get path of uStart.exe
DriveLetter$ = Left(ProgramPath$,2) ;Get drive letter of uStart.exe
Open_Window_0()
LoadData()
RunAppsAuto()
If ListSize(LaunchApp()) = 0
  ;No menuitems yet defined, ask for opening configuration window
  If MessageRequester("No applications defined yet", "There are no applications defined yet in the launching menu! Do you want to set them now?", #PB_MessageRequester_YesNo) = #PB_MessageRequester_Yes
    Open_Window_1()
  EndIf 
EndIf

;{ Eventloop, running ForEver
Repeat
  Event = WaitWindowEvent()
  If Event = #PB_Event_SysTray
    ;***** Systray Events
    If EventType() = #PB_EventType_RightClick
      DisplayPopupMenu(#TrayMenu,	WindowID(Window_0))
    ElseIf EventType() = #PB_EventType_LeftClick
      DisplayPopupMenu(#AppMenu,	WindowID(Window_0))
    EndIf
  ElseIf Event = #PB_Event_CloseWindow
    HideWindow(	Window_0, 1)
  ElseIf Event = #PB_Event_Menu
    ;{***** Menu Events
    Select EventMenu()
      Case 1 ;exit program
        ForEach LaunchApp()
           If LaunchApp()\hWnd <> 0
             If GetMenuItemState(#TrayMenu,4)=1 ;close applications on exit
               ;PostMessage_(LaunchApp()\hWnd,#WM_CLOSE,0,0); close Window
               PostMessage_(LaunchApp()\hWnd,#WM_SYSCOMMAND,#SC_CLOSE,0); close window, simulates pressing close on windows system menu
             EndIf
           EndIf
           DestroyIcon_(LaunchApp()\hIcon) ;free memory used for icons
        Next
        ClearList(LaunchApp()) ; free space occupied by list
        If IsWindow(Window_1) <> 0  ;close settings window when still open
          If IsChanged>0 ; was the entry changed?
            Response = MessageRequesterEx("Save?","Save changes to application entry?",#PB_MessageRequester_YesNoCancel,Window_1)
            If (Response = #PB_MessageRequester_Yes)
              StoreData()
              UpdateTrayMenu()
              SaveData() ;save actual settings
            EndIf
            IsChanged = 0 ; reset changes flag
            If response <> #PB_MessageRequester_Cancel ; close window when not cancelled
              CloseWindow(Window_1)
            EndIf
          Else
            CloseWindow(Window_1)
          EndIf
        EndIf
        CloseWindow(Window_0)
        Break
      Case 2 ;show help
        HideWindow(	Window_0, 0)
      Case 3 ;configure application
        Open_Window_1()
        UpdateTreeDisplay(0) ;display correct tree
        FirstElement(LaunchApp()) ;switch to first element
        CheckEntries()
      Case 4 ;end applications on exit
        If GetMenuItemState(#TrayMenu,4) = 0
          SetMenuItemState(#TrayMenu,4,1)
        Else
          SetMenuItemState(#TrayMenu,4,0)
        EndIf
        If IsWindow(Window_1) <> 0 ;correct displayed gadgets, when window is open
          If GetGadgetState(tree_0)<>-1 ;Set actual position in List
            SelectElement(LaunchApp(),GetGadgetState(tree_0))
          EndIf
          CheckEntries()
        EndIf
        SaveData() ;save changed settings
      Case 5 ;run applications automatically on start
        If GetMenuItemState(#TrayMenu,5) = 0
          SetMenuItemState(#TrayMenu,5,1)
        Else
          SetMenuItemState(#TrayMenu,5,0)
        EndIf
        If IsWindow(Window_1) <> 0 ;correct displayed gadgets, when window is open
          If GetGadgetState(tree_0)<>-1 ;Set actual position in List
            SelectElement(LaunchApp(),GetGadgetState(tree_0))
          EndIf
          CheckEntries()
        EndIf
        SaveData() ;save changed settings
      Default 
      ;***** Call saved applications
        SelectElement(LaunchApp(),EventMenu()-10)
        RunApp() ;run selected application
    EndSelect ;}
  ElseIf Event = #PB_Event_Gadget
    ;***** Gadget Events
    If EventGadget()=#Button_6 ;Exit
      If IsChanged>0 ; was the entry changed?
        Response = MessageRequesterEx("Save?","Save changes to application entry?",#PB_MessageRequester_YesNoCancel,Window_1)
        If (Response = #PB_MessageRequester_Yes)
          StoreData()
          UpdateTrayMenu()
          SaveData() ;save actual settings
        EndIf
        IsChanged = 0 ; reset changes flag
        If response <> #PB_MessageRequester_Cancel ; close window when not cancelled
          CloseWindow(Window_1)
        EndIf
      Else
        CloseWindow(Window_1)
      EndIf
    Else      
      Select EventGadget()
        ;{ Radioevents
        Case #Radio_0
          ;***** Radiobutton: Application
          DisableGadget(#String_0,0)
;          SetGadgetText(#String_0,LaunchApp()\name)
          DisableGadget(#String_1,0)
;          SetGadgetText(#String_1,LaunchApp()\command)
          DisableGadget(#String_2,0)
;          SetGadgetText(#String_2,LaunchApp()\options)
          DisableGadget(#String_3,0)
;          SetGadgetText(#String_3,LaunchApp()\homedir)
          DisableGadget(#Button_7,0)
          DisableGadget(#Button_8,0)
          DisableGadget(#Check_0,0)
;          SetGadgetState(#Check_0,LaunchApp()\run)
          DisableGadget(#Check_1,0)
;          SetGadgetState(#Check_1,LaunchApp()\stop)
        Case #Radio_1
          ;***** Radiobutton: Group
          DisableGadget(#String_0,0)
;          SetGadgetText(#String_0,LaunchApp()\name)
          DisableGadget(#String_1,1)
          SetGadgetText(#String_1,"")
          DisableGadget(#String_2,1)
          SetGadgetText(#String_2,"")
          DisableGadget(#String_3,1)
          SetGadgetText(#String_3,"")
          DisableGadget(#Button_7,1)
          DisableGadget(#Button_8,1)
          DisableGadget(#Check_0,1)
          SetGadgetState(#Check_0,#PB_Checkbox_Unchecked)
          DisableGadget(#Check_1,1)
          SetGadgetState(#Check_1,#PB_Checkbox_Unchecked)
        Case #Radio_2
          ;***** Radiobutton: Separator
          DisableGadget(#String_0,1)
          SetGadgetText(#String_0,"")
          DisableGadget(#String_1,1)
          SetGadgetText(#String_1,"")
          DisableGadget(#String_2,1)
          SetGadgetText(#String_2,"")
          DisableGadget(#String_3,1)
          SetGadgetText(#String_3,"")
          DisableGadget(#Button_7,1)
          DisableGadget(#Button_8,1)
          DisableGadget(#Check_0,1)
          SetGadgetState(#Check_0,#PB_Checkbox_Unchecked)
          DisableGadget(#Check_1,1)
          SetGadgetState(#Check_1,#PB_Checkbox_Unchecked)
        Case #Radio_3
          ;***** Radiobutton: Folder
          DisableGadget(#String_0,0)
 ;         SetGadgetText(#String_0,LaunchApp()\name)
          DisableGadget(#String_1,1)
          SetGadgetText(#String_1,"")
          DisableGadget(#String_2,1)
          SetGadgetText(#String_2,"")
          DisableGadget(#String_3,0)
 ;         SetGadgetText(#String_3,LaunchApp()\homedir)
          DisableGadget(#Button_7,1)
          DisableGadget(#Button_8,0)
          DisableGadget(#Check_0,0)
 ;         SetGadgetState(#Check_0,LaunchApp()\run)
          DisableGadget(#Check_1,1)
          SetGadgetState(#Check_1,#PB_Checkbox_Unchecked)
          ;}
        Case #Button_0
          ;{ Insert application
          If GetGadgetState(tree_0)<>-1 ; Set actual position in List
            SelectElement(LaunchApp(),GetGadgetState(tree_0))
          EndIf
          ActualLevel = LaunchApp()\level ;get level of selected item
          If PreviousElement(LaunchApp())=0
            InsertElement(LaunchApp())
          ElseIf LaunchApp()\level > ActualLevel ;a higher level is visible bofore
            If MessageRequesterEx("Insert application/group/separator","Insert application/group/separator at end of group?",#PB_MessageRequester_YesNo, Window_1)=#PB_MessageRequester_Yes
              ActualLevel = ActualLevel+1
            EndIf
            AddElement(LaunchApp())
          Else ;same or lower level, keep actual level
            AddElement(LaunchApp())
          EndIf
          LaunchApp()\level = ActualLevel
          LastSelectedItem = ListIndex(LaunchApp())
          StoreData() ;save information in List
          UpdateTreeDisplay(LastSelectedItem)
          CheckEntries()
          SaveData() ;save actual settings
          ;} 
        Case #Button_1
          ;{ Add application
          If GetGadgetState(tree_0)<>-1 ; Set actual position in List
            SelectElement(LaunchApp(),GetGadgetState(tree_0))
          EndIf
          If ListIndex(LaunchApp())=-1
            ActualLevel = 0 ;no entries yet
          ElseIf LaunchApp()\type = 1 ;application
            ActualLevel = LaunchApp()\level ;same level as actual entry
          ElseIf LaunchApp()\type = 3 ;separator
            ActualLevel = LaunchApp()\level
          ElseIf LaunchApp()\type = 2 ;group
            If MessageRequesterEx("Add application/group/separator","Add application/group/separator to selected group?",#PB_MessageRequester_YesNo, Window_1)=#PB_MessageRequester_Yes
              ; place entry within group
              ActualLevel = LaunchApp()\level+1
              ; go to end of group
              lastresult = NextElement(LaunchApp())
              While lastresult And (LaunchApp()\level >= ActualLevel)
                lastresult = NextElement(LaunchApp())
              Wend
            Else
              ; place not in group
              ActualLevel = LaunchApp()\level
              ; go to end of group
              lastresult = NextElement(LaunchApp())
              While lastresult And (LaunchApp()\level > ActualLevel)
                lastresult = NextElement(LaunchApp())
              Wend
            EndIf
            If (lastresult<>0)
              PreviousElement(LaunchApp()) ; go back one element
            EndIf
          EndIf
          AddElement(LaunchApp())
          LaunchApp()\level = ActualLevel
          LastSelectedItem = ListIndex(LaunchApp())
          StoreData() ;save information in List
          UpdateTreeDisplay(LastSelectedItem)
          CheckEntries()
          SaveData() ;save actual settings
          ;} 
        Case #Button_2
          ;{ save changes to entry
          If IsChanged>0 ; was the entry changed?
            Response = MessageRequesterEx("Save?","Save changes to application entry and overwrite existing information?"+Chr(13)+"(Cancel = restore old values)",#PB_MessageRequester_YesNoCancel,Window_1)
            If (Response = #PB_MessageRequester_Yes)
              StoreData()
              UpdateTreeDisplay(LastSelectedItem)
              UpdateTrayMenu()
              CheckEntries()
              SaveData() ;save actual settings
              IsChanged = 0; reset changes flag
            EndIf
            If (Response = #PB_MessageRequester_Cancel) ;restore old data
              UpdateValueDisplay(LastSelectedItem)
              IsChanged =0
            EndIf
          EndIf
          ;}
        Case #Button_3
          ;{ Delete Application
          If MessageRequesterEx("Delete","Really delete selected application/group/separator?",#PB_MessageRequester_YesNo, Window_1) = #PB_MessageRequester_Yes
            DestroyIcon_(LaunchApp()\hIcon) ;free memory used for icons
            If LaunchApp()\type = 2
              ;Group
              ActualLevel = LaunchApp()\level
              DeleteElement(LaunchApp(),1)
              While (ListIndex(LaunchApp())<>-1) And (NextElement(LaunchApp())<>0) And (LaunchApp()\level > ActualLevel)
                DeleteElement(LaunchApp(),1)
              Wend
            Else
              DeleteElement(LaunchApp(),1)
            EndIf
            LastSelectedItem = ListIndex(LaunchApp())
            If LastSelectedItem = -1 ;clear fields
              SetGadgetState(#Radio_0,1)
              SetGadgetState(#Radio_1,0)
              SetGadgetState(#Radio_2,0)
              SetGadgetText(#String_0,"")
              SetGadgetText(#String_1,"")
              DisableGadget(#String_1,0)
              SetGadgetText(#String_2,"")
              DisableGadget(#String_2,0)
              SetGadgetText(#String_3,"")
              DisableGadget(#String_3,0)
              DisableGadget(#Button_7,0)
              DisableGadget(#Button_8,0)
              SetGadgetState(#Check_0,0)
            EndIf
            UpdateTreeDisplay(LastSelectedItem)
            SaveData() ;save actual settings
          EndIf
          ;} 
        Case #Button_4
          ;{ Move up
          ActualLevel = LaunchApp()\level ;determine level of actual item
          If LaunchApp()\type=2
          ;{*****actual entry is a group
            LastSelectedItem = ListIndex(LaunchApp()) ;correct the selection to the replaced element
            ActualEntryCount = 0 ;variable for number of entries in the current group
            Repeat ;calculate size of current group i.e. number of entries of current group
              ActualEntrycount = ActualEntrycount +1
            Until (NextElement(LaunchApp())=0) Or (LaunchApp()\level <= ActualLevel)
            SelectElement(LaunchApp(), LastSelectedItem)
            PreviousElement(LaunchApp())
            If (LaunchApp()\level > ActualLevel) Or ((LaunchApp()\type = 2) And (LaunchApp()\level >= ActualLevel))
            ;*****Element before has a higher level = is within a group; or is groupheader
              If MessageRequesterEx("Move into group","Do you want to move into the group?", #PB_MessageRequester_YesNo, Window_1) = #PB_MessageRequester_Yes
              ;*****move into group
                NextElement(LaunchApp())
                LaunchApp()\level = LaunchApp()\level + 1 ;lift entry up one level
                While (NextElement(LaunchApp())<>0) And (LaunchApp()\level <> ActualLevel) ;lift up the whole group one level
                  LaunchApp()\level = LaunchApp()\level + 1 ;lift entry up one level
                Wend
              Else
              ;*****do not move into group
                LastEntryCount = 0 ;variable for number of entries in the group before
                SelectElement(LaunchApp(), LastSelectedItem) ;go back to currently selected entry
                Repeat ;calculate size of previous group i.e. number of entries of previous group
                  LastEntrycount = LastEntrycount +1
                Until (PreviousElement(LaunchApp())=0) Or (LaunchApp()\level <= ActualLevel)
                SelectElement(LaunchApp(), LastSelectedItem) ;go back to currently selected entry
                For i = 1 To LastEntrycount ;repeat as many times as the group before has entries
                  PreviousElement(LaunchApp())
                  LastSelectedItem = ListIndex(LaunchApp()) ; save new position of actual entry
                  *FirstElement = @LaunchApp() ;get pointer to element to be moved
                  For j = 1 To ActualEntrycount ;move element down
                    NextElement(LaunchApp())
                    ListSelection = ListIndex(LaunchApp()) ;correct the selected item
                    *SecondElement = @LaunchApp()
                    SwapElements(LaunchApp(), *FirstElement, *SecondElement)
                    SelectElement(LaunchApp(),ListSelection)
                  Next j
                  SelectElement(LaunchApp(),LastSelectedItem)
                Next i
              EndIf
            ElseIf LaunchApp()\level < ActualLevel
            ;***** Element before has a lower level => is groupheader
              LastSelectedItem = ListIndex(LaunchApp()) ;correct the selection to the replaced element
              *FirstElement = @LaunchApp() ;get pointer to element to be moved
              NextElement(LaunchApp())
              For j = 1 To ActualEntrycount ;move element before group down
                ListSelection = ListIndex(LaunchApp()) ;correct the selected item
                *SecondElement = @LaunchApp()
                LaunchApp()\level = LaunchApp()\level - 1 ;put entry down one level
                SwapElements(LaunchApp(), *FirstElement, *SecondElement)
                SelectElement(LaunchApp(),ListSelection)
                NextElement(LaunchApp())
              Next j
            Else
            ;***** Element before has the same level
              LastSelectedItem = ListIndex(LaunchApp()) ;correct the selection to the replaced element
              *FirstElement = @LaunchApp() ;get pointer to element to be moved
              NextElement(LaunchApp())
              For j = 1 To ActualEntrycount ;move element before group down
                ListSelection = ListIndex(LaunchApp()) ;correct the selected item
                *SecondElement = @LaunchApp()
                SwapElements(LaunchApp(), *FirstElement, *SecondElement)
                SelectElement(LaunchApp(),ListSelection)
                NextElement(LaunchApp())
              Next j
            EndIf ;}
          Else
          ;{***** Anything else than a group
            *FirstElement = @LaunchApp() ;get pointer to actual Element
            PreviousElement(LaunchApp())
            If (LaunchApp()\level > ActualLevel) Or ((LaunchApp()\level = ActualLevel) And (LaunchApp()\type = 2))
            ;*****Element before has a higher level or is groupheader = is within a group
              If MessageRequesterEx("Move into group","Do you want to move into the group?", #PB_MessageRequester_YesNo, Window_1) = #PB_MessageRequester_Yes
              ;*****move into group
                NextElement(LaunchApp()) ;go back to original element
                LastSelectedItem = ListIndex(LaunchApp()) ;correct the selection to the replaced element
                LaunchApp()\level = LaunchApp()\level + 1
              Else
              ;*****do not move into group
                If LaunchApp()\type <> 2 ;not a groupheader, i.e. empty group
                  Repeat ;move element up (before group)
                    LastSelectedItem = ListIndex(LaunchApp()) ;correct the selection to the replaced element
                    *SecondElement = @LaunchApp()
                    SwapElements(LaunchApp(), *FirstElement, *SecondElement)
                    SelectElement(LaunchApp(),LastSelectedItem)
                    PreviousElement(LaunchApp())
                  Until LaunchApp()\level = ActualLevel
                EndIf
                LastSelectedItem = ListIndex(LaunchApp()) ;correct the selection to the replaced element
                *SecondElement = @LaunchApp()
                SwapElements(LaunchApp(), *FirstElement, *SecondElement)
              EndIf  
            Else 
            ;***** Element before has the same or lower level
              If LaunchApp()\level < ActualLevel
                NextElement(LaunchApp())
                LaunchApp()\level = LaunchApp()\level - 1 ;correct level
                PreviousElement(LaunchApp())
              EndIf
              LastSelectedItem = ListIndex(LaunchApp()) ;correct the selection to the replaced element
              *SecondElement = @LaunchApp()
              SwapElements(LaunchApp(), *FirstElement, *SecondElement)              
            EndIf ;}
          EndIf          
          UpdateTreeDisplay(LastSelectedItem)
          SaveData() ;save actual settings
          ;} 
        Case #Button_5
          ;{ Move down
          ActualLevel = LaunchApp()\level
          If LaunchApp()\type=2
          ;{*****actual entry is a group
            LastSelectedItem = ListIndex(LaunchApp()) ;correct the selection to the replaced element
            ActualEntryCount = 0 ;variable for number of entries in the current group
            Repeat ;calculate size of current group i.e. number of entries of current group
              ActualEntrycount = ActualEntrycount + 1
              result = NextElement(LaunchApp())
            Until (result=0) Or (LaunchApp()\level <= ActualLevel)
            If (LaunchApp()\type = 2) And (LaunchApp()\level = ActualLevel) And (result <> 0)
            ;***** Following Element is a groupheader
              If MessageRequesterEx("Move into group","Do you want to move into the group?", #PB_MessageRequester_YesNo, Window_1) = #PB_MessageRequester_Yes
              ;*****move into group
                *FirstElement = @LaunchApp() ;get pointer to element after the group
                For j=1 To ActualEntryCount ;move element after group up
                  PreviousElement(LaunchApp())
                  ListSelection = ListIndex(LaunchApp()) ;correct the selected item
                  LaunchApp()\level = LaunchApp()\level + 1 ;increase level of group
                  *SecondElement = @LaunchApp()
                  SwapElements(LaunchApp(), *FirstElement, *SecondElement)
                  SelectElement(LaunchApp(),ListSelection)
                Next j
                NextElement(LaunchApp()) ;adjust actual active element
                LastSelectedItem = ListIndex(LaunchApp()) ;correct the selection to the replaced element
              Else
              ;*****do not move into group
                NextEntryCount = 0 ;variable for number of entries in the group before
                Repeat ;calculate size of following group i.e. number of entries of previous group
                  NextEntrycount = NextEntrycount +1
                Until (NextElement(LaunchApp())=0) Or (LaunchApp()\level <= ActualLevel)
                For i = 1 To ActualEntrycount ;repeat as many times as the actual group has entries
                  SelectElement(LaunchApp(), LastSelectedItem) ;go back to currently selected entry
                  *FirstElement = @LaunchApp() ;get pointer to element to be moved
                  For j = 1 To (NextEntrycount+ActualEntryCount-1) ;move element down
                    NextElement(LaunchApp())
                    ListSelection = ListIndex(LaunchApp()) ;correct the selected item
                    *SecondElement = @LaunchApp()
                    SwapElements(LaunchApp(), *FirstElement, *SecondElement)
                    SelectElement(LaunchApp(),ListSelection)
                  Next j
                  SelectElement(LaunchApp(),LastSelectedItem)
                Next i
                LastSelectedItem = LastSelectedItem + NextEntryCount ;correct position in the list
              EndIf
            ElseIf (LaunchApp()\level < ActualLevel) Or (result = 0)
            ;***** Following Element has a lower level => end of group
              SelectElement(LaunchApp(),LastSelectedItem) ; correct current selection, go back to beginning of group
              For j=1 To ActualEntryCount
                LaunchApp()\level = LaunchApp()\level - 1 ;get entry down one level
                NextElement(LaunchApp())
              Next j
            Else
            ;***** Following Element has the same level, but is no groupheader
              *FirstElement = @LaunchApp() ;get pointer to element after the group
              For j=1 To ActualEntryCount ;move element after group up
                PreviousElement(LaunchApp())
                ListSelection = ListIndex(LaunchApp()) ;correct the selected item
                *SecondElement = @LaunchApp()
                SwapElements(LaunchApp(), *FirstElement, *SecondElement)
                SelectElement(LaunchApp(),ListSelection)
              Next j
              NextElement(LaunchApp()) ;adjust actual active element
              LastSelectedItem = ListIndex(LaunchApp()) ;correct the selection to the replaced element
            EndIf ;}
          Else
          ;{***** Anything else than a group
            *FirstElement = @LaunchApp() ;get pointer to actual Element
            result = NextElement(LaunchApp())
            If (LaunchApp()\type=2) And result<>0 And (LaunchApp()\level = ActualLevel) ;check whether next entry is a group
            ;*****Next Element is a group
              If MessageRequesterEx("Move into group","Do you want to move into the group?", #PB_MessageRequester_YesNo, Window_1) = #PB_MessageRequester_Yes
                ;move into group
                PreviousElement(LaunchApp())
                LaunchApp()\level = ActualLevel + 1
                NextElement(LaunchApp())
                LastSelectedItem = ListIndex(LaunchApp()) ;correct the selected item
                *SecondElement = @LaunchApp()
                SwapElements(LaunchApp(), *FirstElement, *SecondElement)
              Else ;do not move into group
                Repeat ;move entry down
                  LastSelectedItem = ListIndex(LaunchApp()) ;correct the selected item
                  *SecondElement = @LaunchApp()
                  SwapElements(LaunchApp(), *FirstElement, *SecondElement)
                  SelectElement(LaunchApp(),LastSelectedItem)
                Until (NextElement(LaunchApp())=0) Or (LaunchApp()\level <= ActualLevel)
              EndIf
            ElseIf (LaunchApp()\level < ActualLevel) And result
            ;*****currently at end of group, next one is out of group
              PreviousElement(LaunchApp())
              LaunchApp()\level = ActualLevel - 1 ;correct level
            ElseIf result ; same level
              LastSelectedItem = ListIndex(LaunchApp()) ;correct the selected item
              *SecondElement = @LaunchApp()
              SwapElements(LaunchApp(), *FirstElement, *SecondElement)
            Else ;at end of group and end of list
              LaunchApp()\level = ActualLevel - 1 ;correct level -> one level up
            EndIf ;}
          EndIf
          UpdateTreeDisplay(LastSelectedItem)
          SaveData() ;save actual settings
          ;} 
        Case #Button_7
          ;{ select application
          dir$ = GetGadgetText(#String_1)
          If dir$ = ""
            dir$ = DriveLetter$+"\"
          Else
            dir$ = ConvertToAbsolutePath(GetPathPart(dir$),ProgramPath$)
          EndIf
          file$ = OpenFileRequester("Please choose the application", dir$, "Application (*.exe)| *.exe|All files (*.*)|*.*",1)
          If file$ <> ""
            path$ = ConvertToRelativePath(GetPathPart(file$),ProgramPath$)
            SetGadgetText(#String_1,path$+GetFilePart(file$))
            If GetGadgetText(#String_0) = "" ;fill name if none provided
              SetGadgetText(#String_0, Left(GetFilePart(file$), Len(GetFilePart(file$))-Len(GetExtensionPart(file$))-1))
            EndIf
          EndIf
          ;} 
        Case #Button_8
          ;{ select home directory
          dir$ = GetGadgetText(#String_3)
          If dir$ = ""
            dir$ = DriveLetter$+"\"
          Else
            dir$ = ConvertToAbsolutePath(GetPathPart(dir$),ProgramPath$)
          EndIf
          path$ = PathRequester("Please choose the home directory for your application",dir$)
          If path$ <> ""
            path$ = ConvertToRelativePath(path$,ProgramPath$)
            SetGadgetText(#String_3,path$)
            If (GetGadgetState(#Radio_3) = 1) And (GetGadgetText(#String_0) = "") ;fill name if none provided for folder
              If Right(path$,1) = "\" ;remove last \ if available
                path$ = Left(path$,Len(path$)-1)
              EndIf
              x = CountString(path$,"\")
              For i=1 To x
                path$ = Right(path$,Len(path$)-FindString(path$,"\",1))
              Next
              SetGadgetText(#String_0, "<"+path$+">")
            EndIf
          EndIf
          ;}
        Case #Button_9
          ;{ restore/clear fields
          If GetGadgetText(#Button_9) <> "Restore"
            SetGadgetState(#Radio_0,1)
            SetGadgetState(#Radio_1,0)
            SetGadgetState(#Radio_2,0)
            SetGadgetText(#String_0,"")
            SetGadgetText(#String_1,"")
            DisableGadget(#String_1,0)
            SetGadgetText(#String_2,"")
            DisableGadget(#String_2,0)
            SetGadgetText(#String_3,"")
            DisableGadget(#String_3,0)
            DisableGadget(#Button_7,0)
            DisableGadget(#Button_8,0)
            SetGadgetState(#Check_0,0)
          Else
            UpdateValueDisplay(LastSelectedItem)
            IsChanged = 0
          EndIf
          ;} 
        Case #Button_10
          HideWindow(	Window_0, 1)
        Case Tree_0
          ;{ display information saved according to selected treeitem
          If GetGadgetState(Tree_0)<>-1
            If LastSelectedItem <> GetGadgetState(Tree_0)
              ;delete old selection
              SetGadgetItemState(Tree_0,LastSelectedItem, GetGadgetItemState(Tree_0,LastSelectedItem) & ~#PB_Tree_Selected)
              ; set new selection and display values
              LastSelectedItem = GetGadgetState(Tree_0)
              UpdateValueDisplay(LastSelectedItem)
            EndIf
          EndIf
          ;} 
      EndSelect
      CheckEntries()
    EndIf
  EndIf
ForEver
;}

DataSection
  Trayicon: 
    IncludeBinary "Start16.ico"
  Prefs:
;    IncludeBinary "prefs.ico" ;Iconfinder: "iconset:material-core"
  Exit:
;    IncludeBinary "exit.ico" ;Iconfinder: "iconset:material-core"
  Help:
;    IncludeBinary "help.ico" ;Iconfinder: "iconset:material-core"
  Helptext:
    Data.b 36
    Data.s "uStart - a small and easy usable application launcher for your USB stick"
    Data.s ""
    Data.s "Description:"
    Data.s "  This program is designed with ease of use and filesize in mind. It occupies"
    Data.s "  only some kilobytes of the valuable space on your USB stick."
    Data.s "  It has some features which are very useful for a mobile office on you USB stick."
    Data.s ""
    Data.s "Special features:"
    Data.s "  * Applications could be organized into groups."
    Data.s "  * Separators for better overview"
    ; 10
    Data.s "  * Paths to applications or home directories are stored relative to the location"
    Data.s "    of 'uStart.exe'. '\' for the actual drive is also allowed."
    Data.s "  * Environment variables within paths, applications or options are automatically"
    Data.s "    replaced, e.g. '%windir%' becomes 'C:\windows'."
    Data.s "  * When exiting uStart, the started applications can be optionally closed."
    Data.s "  * After Start you see only an additional trayicon, nothing else."
    Data.s "  * Support for applications, folders and other files that are opened through their"
    Data.s "    associated applications."
    Data.s "  * Applications/Folders/Files could be run/opened automatically on program start"
    Data.s ""
    ; 20
    Data.s "Usage:"
    Data.s "  When left-clicking on the trayicon a popup menu containing the previously"
    Data.s "  defined applications. With clicking on the menuitem you start the application."
    Data.s "  When right-clicking on the trayicon a different popup menu opens, where you"
    Data.s "  can set the applications to appear in the launching menu and configure them."
    Data.s "  Additionally you can set the option whether marked applications that are started"
    Data.s "  should close on exit. Just click on it And you will see a marker, when activated." 
    Data.s "  Opened file types other than .exe are not closed automatically."
    Data.s "  An additional option is the general activation of the automatic run of marked"
    Data.s "  applications. When this option is unchecked it is not possible to activate"
    ; 30
    Data.s "  automatic run when setting the aplications."
    Data.s ""
    Data.s "Free for personal and commercial use."
    Data.s ""
    Data.s "Copyright 2006-2017 Karl Loncarek"
    Data.s "All rights reserved."
EndDataSection
; IDE Options = PureBasic 5.60 (Windows - x86)
; CursorPosition = 565
; FirstLine = 197
; Folding = DKABAAAA+
; EnableAsm
; EnableThread
; EnableXP
; UseIcon = Start.ico
; Executable = ..\uStart.exe
; DisableDebugger
; VD_UsePB_Any = 1
; VD_IncludeBinaryBitmaps = 1